<?php

/* Inclusion de la classe Message
 * NB: ne fonctionne que si le fichier est dans le même répertoire */
require_once("Message.php");

/**
 * Classe de stockage de message. Utilise un tableau
 * de messages qui est sérialisé et enregistré dans
 * un fichier après chaque requête, ce qui permet
 * la persistance des données.
 *
 * Pas prévue pour être utilisée telle quelle en production
 * sur un site un peu gros:
 *     1. le serveur passe son temps à lire le fichier intégralement
 *        et à le réécrire intégralement: probablement assez inefficace
 *     2. modifications concurrentes pas du tout gérées:
 *        aucune robustesse
 * Mais si on prend soin de ces deux aspects, utiliser des fichiers
 * plutôt qu'une base de données est parfaitement possible, à moins
 * que votre site soit vraiment énorme.
 */
class MessageStorage {

	/** La liste de messages. */
	protected $db;

	/** Le fichier utilisé pour sauvegarder les données. */
	protected $file;

	/** Construit une nouvelle instance. */
	public function __construct() {
		$this->file = $_SERVER['TMPDIR'].'/messagedb.txt';
		if (file_exists($this->file)) {
			/* on a sérialisé un tableau contenant la base à l'indice db
			 * et le prochain identifiant à l'indice nextId */
			$storedData = unserialize(base64_decode(file_get_contents($this->file)));
			$this->db = $storedData['db'];
			$this->nextId = $storedData['nextId'];
		} else {
			/* Le fichier n'existe pas, on crée une base avec des messages bidon */
			$this->reinit();
		}
	}

	/** Sérialise et stocke la base avant de détruire l'instance. */
	public function __destruct() {
		$dataToBeStored = array('db' => $this->db, 'nextId' => $this->nextId);
		file_put_contents($this->file, base64_encode(serialize($dataToBeStored)));
	}

	/**
	 * Initialise la base avec un tableau vide.
	 */
	public function init() {
		$this->db = array();
		$this->nextId = 1;
	}

	/**
	 * Réinitialise la base avec des messages bidon.
	 */
	public function reinit() {
		$this->init();
		$this->create(new Message('toto', 'C’est mon premier commentaire.', 'example.com', new DateTime('2008-07-01T22:35:17.03+08:00')));
		$this->create(new Message('Jean-Mimi', 'J’adore ce site !', 'example.com', new DateTime('2013-11-11T12:07:20.45+02:00')));
	}

	/**
 	 * Renvoie un tableau contenant tous les messages
 	 * d'identifiant compris entre $startId (inclus) et $endId (exclus).
 	 */
	public function readBetween($startId, $endId) {
		$res = array();
		foreach ($this->db as $id => $msg) {
			if (intval($startId) <= intval($id) && intval($id) < intval($endId)) {
				$res[$id] = $msg;
			}
		}
		return $res;
	}

	/**
 	 * Renvoie un tableau contenant tous les messages
 	 * d'identifiant strictement supérieur à $previousId.
 	 */
	public function readAfter($previousId) {
		return $this->readBetween($previousId+1, $this->nextId);
	}

	/**
 	 * Renvoie un tableau contenant les $nb derniers messages.
 	 */
	public function readLast($nb) {
		return $this->readBetween($this->nextId - $nb, $this->nextId);
	}

	/**
 	 * Renvoie un tableau contenant tous les messages
 	 * de la base.
 	 */
	public function readAll() {
		return $this->db;
	}

	/**
 	 * Ajoute un nouveau message à la base. Renvoie
 	 * l'identifiant du message ajouté.
 	 */
	public function create(Message $msg) {
		$id = strval($this->nextId);  /* J'utilise des chaînes pour les identifiants.
		                               * C'est plus propre, car les valeurs passées
		                               * dans l'URL ou en POST sont toujours des chaînes.
		                               * Ne manipuler que des chaînes partout évite la
		                               * tentation d'utiliser == au lieu de ===. */
		if (key_exists($id, $this->db))
			throw new Exception('The next id is already used');
		$this->db[$id] = $msg;
		$this->nextId++;
		return $id;
	}
}

?>
